spec_autoconf=(autoconf-2.69.tar.xz https://ftp.gnu.org/gnu/autoconf/autoconf-2.69.tar.xz 64ebcec9f8ac5b2487125a86a7760d2591ac9e1d3dbd59489633f9de62a57684)
spec_automake=(automake-1.16.1.tar.gz https://ftp.gnu.org/gnu/automake/automake-1.16.1.tar.xz 5d05bb38a23fd3312b10aea93840feec685bdf4a41146e78882848165d3ae921)
spec_boost=(boost_1_74_0.tar.bz2 https://boostorg.jfrog.io/artifactory/main/release/1.74.0/source/boost_1_74_0.tar.bz2 83bfc1507731a0906e387fc28b7ef5417d591429e51e788417fe9ff025e116b1)
spec_cmake=(cmake-3.20.1.tar.gz https://cmake.org/files/v3.20/cmake-3.20.1.tar.gz 3f1808b9b00281df06c91dd7a021d7f52f724101000da7985a401678dfe035b0)
spec_cmark=(cmark-0.29.0.tar.gz https://github.com/commonmark/cmark/archive/0.29.0.tar.gz 2558ace3cbeff85610de3bda32858f722b359acdadf0c4691851865bb84924a6)
spec_curl=(curl-7.60.0.tar.xz https://curl.haxx.se/download/curl-7.60.0.tar.xz 8736ff8ded89ddf7e926eec7b16f82597d029fc1469f3a551f1fafaac164e6a0)
spec_flac=(flac-1.3.3.tar.xz http://downloads.xiph.org/releases/flac/flac-1.3.3.tar.xz 213e82bd716c9de6db2f98bcadbc4c24c7e2efe8c75939a1a84e28539c4e1748)
spec_gettext=(gettext-0.21.tar.gz https://ftp.gnu.org/gnu/gettext/gettext-0.21.tar.xz d20fcbb537e02dcf1383197ba05bd0734ef7bf5db06bdb241eb69b7d16b73192)
spec_gmp=(gmp-6.1.2.tar.xz https://gmplib.org/download/gmp/gmp-6.1.2.tar.xz 87b565e89a9a684fe4ebeeddb8399dce2599f9c9049854ca8c0dfbdea0e21912)
spec_libiconv=(libiconv-1.16.tar.gz https://ftp.gnu.org/gnu/libiconv/libiconv-1.16.tar.gz e6a1b1b589654277ee790cce3734f07876ac4ccfaecbee8afa0b649cf529cc04)
spec_ogg=(libogg-1.3.4.tar.xz http://downloads.xiph.org/releases/ogg/libogg-1.3.4.tar.xz c163bc12bc300c401b6aa35907ac682671ea376f13ae0969a220f7ddf71893fe)
spec_pcre2=(pcre2-10.37.tar.bz2 https://ftp.pcre.org/pub/pcre/pcre2-10.37.tar.bz2 4d95a96e8b80529893b4562be12648d798b957b1ba1aae39606bbc2ab956d270)
spec_pkgconfig=(pkg-config-0.29.2.tar.gz https://pkg-config.freedesktop.org/releases/pkg-config-0.29.2.tar.gz 6fc69c01688c9458a57eb9a1664c9aba372ccda420a02bf4429fe610e7e7d591)
spec_vorbis=(libvorbis-1.3.7.tar.xz http://downloads.xiph.org/releases/vorbis/libvorbis-1.3.7.tar.xz b33cc4934322bcbf6efcbacf49e3ca01aadbea4114ec9589d1b1e9d20f72954b)
spec_zlib=(zlib-1.2.11.tar.xz https://www.zlib.net/zlib-1.2.11.tar.gz c3e5e9fdd5004dcb542feda5ee4f0ff0744628baf8ed2dd5d66f8ca1197cb1a1)

spec_qtbase=(qtbase-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qtbase-everywhere-src-5.15.2.tar.xz 909fad2591ee367993a75d7e2ea50ad4db332f05e1c38dd7a5a274e156a4e0f8)
spec_qtimageformats=(qtimageformats-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qtimageformats-everywhere-src-5.15.2.tar.xz bf8285c7ce04284527ab823ddc7cf48a1bb79131db3a7127342167f4814253d7)
spec_qtmultimedia=(qtmultimedia-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qtmultimedia-everywhere-src-5.15.2.tar.xz 0c3758810e5131aabcf76e4965e4c18b8911af54d9edd9305d2a8278d8346df5)
spec_qtsvg=(qtsvg-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qtsvg-everywhere-src-5.15.2.tar.xz 8bc3c2c1bc2671e9c67d4205589a8309b57903721ad14c60ea21a5d06acb585e)
spec_qttools=(qttools-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qttools-everywhere-src-5.15.2.tar.xz c189d0ce1ff7c739db9a3ace52ac3e24cb8fd6dbf234e49f075249b38f43c1cc)
spec_qttranslations=(qttranslations-everywhere-src-5.15.2.tar.xz https://download.qt.io/official_releases/qt/5.15/5.15.2/submodules/qttranslations-everywhere-src-5.15.2.tar.xz d5788e86257b21d5323f1efd94376a213e091d1e5e03b45a95dd052b5f570db8)
